 /*******************************************************************
 *
 *  DESCRIPTION: Atomic Model Queue
 *
 *  AUTHOR: Misagh Tavanpour
 *
 *  DATE: 18/11/2012
 *
 *******************************************************************/
 #include "Queue2.h"      // class Queue
 #include "message.h"    // class ExternalMessage, InternalMessage
 #include <iostream>
 #include <fstream>
 ofstream UQI2 ("UE2QIn.txt");
 ofstream UQO2 ("UE2QOut.txt");

 Queue2::Queue2( const string &name ) : Atomic( name )
 , In( addInputPort( "In" ) )
 , Req( addInputPort( "Req" ) )
 , Out( addInputPort( "Out" ) )
 , ProcessTime (00,00,00,00)
 {
 }

 Model &Queue2::initFunction()
 {
   Request = 0;
   Qlen = 0;
   state = Idle;
   elements.erase( elements.begin(), elements.end() ) ;
   passivate();
   return *this ;
 }

 Model &Queue2::externalFunction( const ExternalMessage &msg )
 { UQI2 <<msg.time()<<" "<<msg.value()<<"\n";
   if( msg.port() != Req ) // Input from In1 or In2 or In3
	 {
	   elements.push_back( msg.value() ) ; //Store Input value in Queue
	   state = Push;
	   Qlen++;
	   holdIn( Atomic::active, ProcessTime);
	 }
     else if (elements.size() > 0)  //So msg.port() == Req  ===> Pop from Queue (Check that the Queue is not empty)
          	 {
    	       Request = 1;
    	       state = Pop;
    	       holdIn( Atomic::active, ProcessTime);
      	     }
             else if (elements.size() == 0)
            	     {      // Here this two condition are True :(msg.port() == Req) & (elements.size() == 0)
            	       Request = 1;   // Although Queue is empty but remember that server is waiting for input.
            	       passivate();   // If the queue is empty then passivate.
            	     }
   return *this;
 }

 Model &Queue2::outputFunction( const InternalMessage &msg )
 {
   if ((state == Pop) && (Request == 1))
	  {
	     sendOutput( msg.time(), Out, elements.front());  // Send out data from Queue
	     UQO2 <<msg.time()<<" Out "<<elements.front()<<"\n";
	  }
   return *this ;
 }

 Model &Queue2::internalFunction( const InternalMessage & )
 {
   switch (state){
   				    case Idle:
   				    	       if (Request == 1) state = Pop;
   				    	         else passivate();
   					           break;

   				    case Push:
 			    	            state = Idle;
   					            break;

   				    case Pop:
   				    	       elements.pop_front();
   				    	       Qlen--;
   				    	       Request = 0;
   				    	       state = Idle;
   					           break;
           		 }
   return *this;
 }

 Queue2::~Queue2()
 {
 }
